@@ -2,7 +2,7 @@
  *
  * Information about ssh channels.
  *
- * $Id: channel.h,v 1.34 1999/06/11 23:25:44 nisse Exp $
+ * $Id: channel.h,v 1.38 1999/06/30 20:43:49 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -57,6 +57,7 @@
 /* GABA:
    (class
      (name ssh_channel)
+     (super flow_controlled)
      (vars
        ; Remote channel number 
        (channel_number simple UINT32)
@@ -151,6 +152,11 @@
        ; Channels are indexed by local number
        (channels pointer (object ssh_channel) used_channels)
 
+       ; Global requests that we support
+       (global_requests object alist)
+       ; Channel types that we can open
+       (channel_types object alist)
+       
        ; Allocation of local channel numbers is managed using the same
        ; method as is traditionally used for allocation of unix file 
        ; descriptors.
@@ -161,6 +167,10 @@
        (used_channels simple UINT32)
        (max_channels simple UINT32) ; Max number of channels allowed 
 
+       ; Forwarded TCP ports
+       (local_ports struct object_queue)
+       (remote_ports struct object_queue)
+       
        ; Global requests that we have received, and should reply to
        ; in the right order
        (active_global_requests struct object_queue)
@@ -218,22 +228,6 @@
        (connection object ssh_connection)))
 */
 
-/* xxCLASS:
-   (class
-     (name channel_open_response)
-     (vars
-       (response method int
-                "struct ssh_channel *channel"
-                "UINT32 error"
-		; FIXME: Use an lsh_string for error messages
-                "char *error_msg"
-                "struct lsh_string *args")
-       (connection object ssh_connection)
-       (remote_channel_number simple UINT32)
-       (window_size simple UINT32)
-       (max_packet simple UINT32)))
-*/
-
 #define CHANNEL_OPEN_CALLBACK(c, ch, e, m, a) \
   (c)->response((c), (ch), (e), (m), (a))
 
@@ -305,6 +299,8 @@ struct lsh_string *format_channel_failure(UINT32 channel);
 struct lsh_string *prepare_window_adjust(struct ssh_channel *channel,
 					 UINT32 add);
 
+int channel_start_receive(struct ssh_channel *channel);
+
 struct lsh_string *prepare_channel_open(struct channel_table *table,
 					int type,
 					struct ssh_channel *channel,
@@ -330,8 +326,11 @@ struct lsh_string *channel_transmit_extended(struct ssh_channel *channel,
 					     UINT32 type,
 					     struct lsh_string *data);
 
+extern struct command connection_service;
+
+#if 0
 struct command *make_connection_service(struct alist *global_requests,
 					struct alist *channel_types);
-
+#endif
 
 #endif /* LSH_CHANNEL_H_INCLUDED */
