@@ -1,6 +1,6 @@
 /* client.c
  *
- * $Id: client.c,v 1.66 1999/05/26 17:23:12 nisse Exp $ */
+ * $Id: client.c,v 1.67 1999/06/14 01:23:49 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -408,6 +408,10 @@ struct ssh_channel *make_client_session(struct io_fd *in,
   self->out = out;
   self->err = err;
 
+  /* Flow control */
+  out->buffer->report = &self->super.super;
+  err->buffer->report = &self->super.super;
+  
   self->exit_status = exit_status;
   
   return &self->super;
