@@ -1,6 +1,6 @@
 /* command.h
  *
- * $Id: command.h,v 1.23 1999/06/11 20:46:31 nisse Exp $ */
+ * $Id: command.h,v 1.29 1999/06/30 20:54:27 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -45,6 +45,13 @@
        (c method int "struct lsh_object *result")))
 */
 
+/* GABA:
+   (class
+     (name command_exception)
+     (vars
+       (e method int UINT32)))
+*/
+
 /* GABA:
    (class
      (name command)
@@ -65,8 +72,11 @@
 
 #define COMMAND_CALL(f, a, c) \
   ((f)->call((f), (struct lsh_object *) (a), (c)))
+
+/* NOTE: If R == NULL, don't call the continuation, _but_ the argument
+ * V must still be evaluated. */
 #define COMMAND_RETURN(r, v) \
-  ((r) ? ((r)->c((r), (struct lsh_object *) (v))) : LSH_OK | LSH_GOON)
+  ((r) ? ((r)->c((r), (struct lsh_object *) (v))) : ((void)(v), LSH_OK | LSH_GOON))
 #define COMMAND_SIMPLE(f, a) \
   ((f)->call_simple((f), (struct lsh_object *)(a)))
 
@@ -77,9 +87,13 @@ int do_call_simple_command(struct command *s,
 #define STATIC_COMMAND_SIMPLE(f) \
 { { STATIC_HEADER, do_call_simple_command }, f}
 
+#define STATIC_COMMAND(f) { STATIC_HEADER, f }
+
 struct command *make_parallell_progn(struct object_list *body);
 extern struct command_simple progn_command;
 
+extern struct command_continuation discard_continuation;
+
 /* GABA:
    (class
      (name command_frame)
@@ -157,6 +171,12 @@ make_collect_state_3(struct collect_info_3 *info,
 { { { STATIC_HEADER, do_call_simple_command }, do_collect_1}, \
   make_collect_state_1, next }
 
+#if 0
+#define STATIC_COLLECT_1_FINAL(f) \
+{ { { STATIC_HEADER, do_call_simple_command }, do_collect_1}, \
+  f, NULL }
+#endif
+
 #define STATIC_COLLECT_2(next) \
 { STATIC_HEADER, make_collect_state_2, next }
 
@@ -174,6 +194,16 @@ extern struct command_simple command_unimplemented;
 
 struct command command_die_on_null;
 
+
+struct command *make_trace(const char *name, struct command *real);
+struct lsh_object *collect_trace(const char *name, struct lsh_object *real);
+
+#if 1
+#define MAKE_TRACE(name, real) (collect_trace((name), (real)))
+#else
+#define MAKE_TRACE(name, real) (real)
+#endif
+
 /* The GABA_* macros are used by automatically generated evaluation code */
 
 struct command_continuation *
@@ -194,18 +224,18 @@ struct command *make_command_K_1(struct lsh_object *x);
 #define GABA_APPLY_K_1(x) ((struct lsh_object *) make_command_K_1(x))
 
 extern struct collect_info_1 command_S;
-/* extern struct collect_info_2 collect_info_S_2; */
+extern struct collect_info_2 collect_info_S_2; 
 
 struct command *make_command_S_2(struct command *f,
 				 struct command *g);
 
 #define GABA_VALUE_S (&command_S.super.super.super)
 #define GABA_APPLY_S_1(f) (make_collect_state_1(&command_S, (f)))
-#define GABA_APPLY_S_2(f, g) (collect_S_2(NULL, (f), (g)))
+#define GABA_APPLY_S_2(f, g) (collect_S_2(&collect_info_S_2, (f), (g)))
 
 extern struct collect_info_1 command_Sp;
 extern struct collect_info_2 collect_info_Sp_2;
-/* extern struct collect_info_3 collect_info_Sp_3; */
+extern struct collect_info_3 collect_info_Sp_3;
 
 struct command *make_command_Sp_3(struct command *c,
 				  struct command *f,
@@ -223,10 +253,10 @@ struct lsh_object *collect_Sp_3(struct collect_info_3 *info,
 #define GABA_APPLY_Sp_1(c) (make_collect_state_1(&command_Sp, (c)))
 #define GABA_APPLY_Sp_2(c, f) \
   (make_collect_state_2(&collect_info_Sp_2, (c), (f)))
-#define GABA_APPLY_Sp_3(c, f, g) (collect_Sp_3(NULL, (c), (f), (g)))
+#define GABA_APPLY_Sp_3(c, f, g) (collect_Sp_3(&collect_info_Sp_3, (c), (f), (g)))
 
 extern struct collect_info_1 command_B;
-/* extern struct collect_info_2 collect_info_B_2; */
+extern struct collect_info_2 collect_info_B_2; 
 
 struct command *make_command_B_2(struct command *f,
 				 struct command *g);
@@ -236,7 +266,7 @@ struct lsh_object *collect_B_2(struct collect_info_2 *info,
 
 #define GABA_VALUE_B (&command_B.super.super)
 #define GABA_APPLY_B_1(f) (make_collect_state_1(&command_B, (f)))
-#define GABA_APPLY_B_2(f, g) (collect_B_2(NULL, (f), (g)))
+#define GABA_APPLY_B_2(f, g) (collect_B_2(&collect_info_B_2, (f), (g)))
 
 extern struct collect_info_1 command_Bp;
 extern struct collect_info_2 collect_info_Bp_2;
@@ -254,10 +284,10 @@ struct lsh_object *collect_Bp_3(struct collect_info_3 *info,
 #define GABA_APPLY_Bp_1(c) (make_collect_state_1(&command_Bp, (c)))
 #define GABA_APPLY_Bp_2(c, f) \
   (make_collect_state_2(&collect_info_Bp_2, (c), (f)))
-#define GABA_APPLY_Bp_3(c, f, g) (collect_Bp_3(NULL, (c), (f), (g)))
+#define GABA_APPLY_Bp_3(c, f, g) (collect_Bp_3(&collect_info_Bp_3, (c), (f), (g)))
 
 extern struct collect_info_1 command_C;
-/* extern struct collect_info_2 collect_info_C_2; */
+extern struct collect_info_2 collect_info_C_2; 
 
 struct command *
 make_command_C_2(struct command *f,
@@ -269,11 +299,11 @@ collect_C_2(struct collect_info_2 *info,
 
 #define GABA_VALUE_C (&command_C.super.super.super)
 #define GABA_APPLY_C_1(f) (make_collect_state_1(&command_C, (f)))
-#define GABA_APPLY_C_2(f, y) (collect_C_2(NULL, (f), (y)))
+#define GABA_APPLY_C_2(f, y) (collect_C_2(&collect_info_C_2, (f), (y)))
 
 extern struct collect_info_1 command_Cp;
 extern struct collect_info_2 collect_info_Cp_2;
-/* extern struct collect_info_3 collect_info_Cp_3; */
+extern struct collect_info_3 collect_info_Cp_3;
 
 struct command *
 make_command_Cp_3(struct command *c,
@@ -289,7 +319,7 @@ collect_Cp_3(struct collect_info_3 *info,
 #define GABA_APPLY_Cp_1(c) (make_collect_state_1(&command_Cp, (c)))
 #define GABA_APPLY_Cp_2(c, f) \
   (make_collect_state_2(&collect_info_Cp_2, (c), (f)))
-#define GABA_APPLY_Cp_3(c, f, y) (collect_Cp_3(NULL, (c), (f), (y)))
+#define GABA_APPLY_Cp_3(c, f, y) (collect_Cp_3(&collect_info_Cp_3, (c), (f), (y)))
      
 
 #endif /* LSH_COMMAND_H_INCLUDED */ 
