@@ -1,6 +1,6 @@
 /* io_commands.c
  *
- * $Id: io_commands.c,v 1.8 1999/06/11 18:24:55 nisse Exp $
+ * $Id: io_commands.c,v 1.14 1999/06/30 21:00:30 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -125,6 +125,7 @@ do_listen_connection(struct command *s,
   if (!address_info2sockaddr_in(&sin, address))
     return COMMAND_RETURN(c, NULL);
 
+  /* FIXME: Asyncronous dns lookups should go here */
   return COMMAND_RETURN(c, io_listen
 			(self->backend, &sin,
 			 make_listen_command_callback
@@ -147,12 +148,12 @@ struct command *make_listen_command(struct command *callback,
 
 static struct lsh_object *
 collect_listen(struct collect_info_2 *info,
-	       struct lsh_object *b,
-	       struct lsh_object *c)
+	       struct lsh_object *a,
+	       struct lsh_object *b)
 {
-  CAST(command, callback, c);
+  CAST_SUBTYPE(command, callback, a);
   CAST(io_backend, backend, b);
-  assert(!info);
+  assert(!info->next);
 
   return &make_listen_command(callback, backend)->super;
 }
@@ -163,6 +164,8 @@ STATIC_COLLECT_2_FINAL(collect_listen);
 struct collect_info_1 listen_command =
 STATIC_COLLECT_1(&collect_info_listen_2);
 
+
+
 /* GABA:
    (class
      (name connect_command_callback)
@@ -222,10 +225,63 @@ static int do_connect(struct io_backend *backend,
   return LSH_OK | LSH_GOON;
 }
 
-/* Simple connect function taking port only as argument. Also used for
- * listen.
+
+/* Connect variant, taking a connection object as argument (used for
+ * rememembering the connected fd).
  *
- * (connect address) */
+ * (connect backend port connection) -> fd */
+
+/* GABA:
+   (class
+     (name connect_port)
+     (super command)
+     (vars
+       (backend object io_backend)
+       (target object address_info)))
+*/
+
+static int do_connect_port(struct command *s,
+			   struct lsh_object *x,
+			   struct command_continuation *c)
+{
+  CAST(connect_port, self, s);
+  CAST(ssh_connection, connection, x);
+  
+  return do_connect(self->backend, self->target, connection->resources, c);
+}
+
+
+struct command *make_connect_port(struct io_backend *backend,
+				  struct address_info *target)
+{
+  NEW(connect_port, self);
+  self->super.call = do_connect_port;
+  self->backend = backend;
+  self->target = target;
+
+  return &self->super;
+}
+
+static struct lsh_object *
+collect_connect_port(struct collect_info_2 *info,
+		     struct lsh_object *a,
+		     struct lsh_object *b)
+{
+  CAST(io_backend, backend, a);
+  CAST(address_info, target, b);
+  assert(!info->next);
+  assert(backend);
+  assert(target);
+  
+  return &make_connect_port(backend, target)->super;
+}
+
+static struct collect_info_2 collect_info_connect_port_2 =
+STATIC_COLLECT_2_FINAL(collect_connect_port);
+
+struct collect_info_1 connect_with_port =
+STATIC_COLLECT_1(&collect_info_connect_port_2);
+
 
 /* GABA:
    (class
@@ -236,6 +292,11 @@ static int do_connect(struct io_backend *backend,
        (resources object resource_list)))
 */
 
+/* Simple connect function taking port only as argument. Also used for
+ * listen.
+ *
+ * (connect address) */
+
 static int do_simple_connect(struct command *s,
 			     struct lsh_object *a,
 			     struct command_continuation *c)
@@ -259,6 +320,38 @@ make_simple_connect(struct io_backend *backend,
   return &self->super;
 }
 
+
+/* (connect port connection) */
+/* GABA:
+   (class
+     (name connect_connection)
+     (super command)
+     (vars
+       (backend object io_backend)))
+*/
+
+static int do_connect_connection(struct command *s,
+				 struct lsh_object *x,
+				 struct command_continuation *c)
+{
+  CAST(connect_connection, self, s);
+  CAST(ssh_connection, connection, x);
+
+  return
+    COMMAND_RETURN(c,
+		   make_simple_connect(self->backend, connection->resources));
+}
+
+struct command *make_connect_connection(struct io_backend *backend)
+{
+  NEW(connect_connection, self);
+  self->super.call = do_connect_connection;
+  self->backend = backend;
+
+  return &self->super;
+}
+
+
 static int do_simple_listen(struct command *s,
 			    struct lsh_object *a,
 			    struct command_continuation *c)
