@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: werror.h,v 1.18 1999/04/07 19:40:33 nisse Exp $ */
+ * $Id: werror.h,v 1.19 1999/06/22 22:22:48 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -65,6 +65,9 @@ void werror(const char *format, ...);
 void debug(const char *format, ...);
 void verbose(const char *format, ...);
 
+/* FIXME: Use a separate flag to enable tracing */
+#define trace debug
+
 void fatal(const char *format, ...) NORETURN;
 
 #endif /* LSH_ERROR_H_INCLUDED */
